import numpy as np


def bond_centered_amn_proj(seedname="m1", dirc="./", n_wf=12, n_bands=22):
    """
    Transforms atom-centered wannier functions into bond-centered ones
    by the means of modifying the <seedname>.amn file using pre-defined
    rotation matrix. This is done in-place, and requires only the old
    <seedname>.amn file and a <seedname>.win file for a list of k-points.

    Parameters
    ----------
    seedname : string
        seedname for the W90 calculation
    dirc : string
        directory hosting the <seedname>.amn and kps.txt files
    n_wf : int
        number of wannier functions
    n_bands : int
        number of bands

    Returns
    -------
    None
    """

    with open(dirc + f"{seedname}.win") as file:
        kps_txt = file.read().split("begin kpoints")[1].split("\n")[1:-2]
        kps = []
        for i in kps_txt:
            kps.append([float(k) for k in i.split()])

    n_k = len(kps)
    amn_head = np.genfromtxt(dirc + f"{seedname}.amn", dtype="str", max_rows=1)
    if amn_head[2] == "0":
        print("amn_proj already done")
    amn_data = np.loadtxt(dirc + f"{seedname}.amn", skiprows=2)
    save_shape = amn_data.shape
    assert amn_data.shape[0] == n_k * n_bands * n_wf
    amn_data = amn_data.reshape((n_k, n_wf, n_bands, 5))

    projmat_amn = amn_data[:, :, :, 3] + 1j * amn_data[:, :, :, 4]

    # create rotation matrix in a very explicit way
    dimer_dir = [0, 0, 1]
    rotmat = np.array([np.eye(n_wf, dtype=complex)] * n_k)

    for i in range(n_k):
        starting_phase = 1j * np.pi / 4
        phase = 1j * 2 * np.pi * ((np.dot(kps[i], dimer_dir)))
        delta = -1 / 4
        phase00 = delta * phase
        phase03 = -delta * phase
        phase30 = (delta - 1 / 2) * phase
        phase33 = (-delta - 1 / 2) * phase

        rotmat[i, 0, 0] = np.exp(+1 * starting_phase + phase00)
        rotmat[i, 1, 1] = np.exp(+1 * starting_phase + phase00)
        rotmat[i, 2, 2] = np.exp(+1 * starting_phase + phase00)
        rotmat[i, 3, 3] = np.exp(+1 * starting_phase + phase33)
        rotmat[i, 4, 4] = np.exp(+1 * starting_phase + phase33)
        rotmat[i, 5, 5] = np.exp(+1 * starting_phase + phase33)

        rotmat[i, 6, 6] = rotmat[i, 0, 0]
        rotmat[i, 7, 7] = rotmat[i, 1, 1]
        rotmat[i, 8, 8] = rotmat[i, 2, 2]
        rotmat[i, 9, 9] = rotmat[i, 3, 3]
        rotmat[i, 10, 10] = rotmat[i, 4, 4]
        rotmat[i, 11, 11] = rotmat[i, 5, 5]

        rotmat[i, 0, 3] = np.exp(-1 * starting_phase + phase03)
        rotmat[i, 1, 4] = np.exp(-1 * starting_phase + phase03)
        rotmat[i, 2, 5] = np.exp(-1 * starting_phase + phase03)
        rotmat[i, 3, 0] = np.exp(-1 * starting_phase + phase30)
        rotmat[i, 4, 1] = np.exp(-1 * starting_phase + phase30)
        rotmat[i, 5, 2] = np.exp(-1 * starting_phase + phase30)

        rotmat[i, 6, 9] = rotmat[i, 0, 3]
        rotmat[i, 7, 10] = rotmat[i, 1, 4]
        rotmat[i, 8, 11] = rotmat[i, 2, 5]
        rotmat[i, 9, 6] = rotmat[i, 3, 0]
        rotmat[i, 10, 7] = rotmat[i, 4, 1]
        rotmat[i, 11, 8] = rotmat[i, 5, 2]

    nrml = 1 / np.sqrt(2)
    rotmat = nrml * rotmat

    projmat_amn_bab = np.einsum("kab,kbc->kac", rotmat, projmat_amn)

    # save data
    amn_data[:, :, :, 3] = np.real(projmat_amn_bab)
    amn_data[:, :, :, 4] = np.imag(projmat_amn_bab)

    save_amn_data = amn_data.reshape(save_shape)

    start_str = f" Created on 0 at  0: 0: 0  \n          {n_bands}         {n_k}          {n_wf}"
    fmt = "%d", "%d", "%d", "%1.12f", "%1.12f"
    np.savetxt(
        dirc + f"{seedname}.amn", save_amn_data, fmt=fmt, header=start_str, comments=""
    )

    return None


dirc = "./"

bond_centered_amn_proj("m1", dirc, 12, 12)
